/// Plugins for Entice Designer
// www.dprogramming.com/entice.php
/// $(B See readme.txt)
module enticeplugin;


extern(Windows)
{
	alias void function(EnticeFunctions* efuncs, int id) EnticeIdCallback;
	
	alias int function(int pluginid, char* msg, int msglen, int msgtype) EnticeShowMsgProc;
	
	// tomenuid = 0 is normal add, tomenuid = -1 to remove, otherwise specifies parent menu id to add to.
	alias int function(int pluginid, char* name, int namelen, int menuid, int tomenuid, EnticeIdCallback callback) EnticeRegisterMenuItemProc;
	
	// keys = same deal as DFL's Keys enum. Bits: shift = 0x10000, control = 0x20000, alt = 0x40000.
	// Ctrl+A = ('A' | 0x20000); 0 = '0'; F1 = 112;
	alias int function(int pluginid, int keys, int shortcutid, EnticeIdCallback callback) EnticeRegisterShortcutProc;
	
	alias int function(int pluginid, int featureid, void* getinfo, size_t getinfosize) EnticeGetFeatureProc;
	
	alias int function(int pluginid, int configtype, char* name, int namelen, char* value, int valuelen) EF_Config_GetConfigProc;
	alias int function(int pluginid, int configtype, char* name, int namelen, char* value, int valuelen) EF_Config_SetConfigProc;
}


/// Functions provided to the plugin by Entice Designer.
struct EnticeFunctions
{
	int id; /// This plugin ID.
	int more; ///
	
	
	/**
		Show a message: int showMsg(int pluginid, char* msg, int msglen, int msgtype);
		Params:
			pluginid = this plugin ID.
			msg = pointer to the message.
			msglen = length of bytes at msg.
			msgtype = one of SHOW_MSG_* constants.
	**/
	EnticeShowMsgProc showMsg;
	
	/**
		Interface with menus: int registerMenuItem(int pluginid, char* name, int namelen, int menuid, int tomenuid, EnticeIdCallback callback);
		Params:
			pluginid = this plugin ID.
			name = pointer to the name displayed on the menu item.
			namelen = length of bytes at name.
			menuid = menu ID unique to this plugin. Must be positive.
			tomenuid = the parent menu to add to, -1 to remove, or one of EMENU_* constants.
			callback = called back on click: menuCallback(EnticeFunctions* efuncs, int menuid);
		Returns: 0 on failure.
	**/
	EnticeRegisterMenuItemProc registerMenuItem;
	
	/**
		Interface with shortcut keys: int registerShortcut(int pluginid, int keys, int shortcutid, EnticeIdCallback callback);
		Params:
			pluginid = this plugin ID.
			keys = key code.
			shortcutid = shortcut ID unique to this plugin. Must be positive.
			callback = called back upon user pressing the shortcut: shortcutCallback(EnticeFunctions* efuncs, int shortcutid);
		Returns: 0 on failure.
	**/
	EnticeRegisterShortcutProc registerShortcut;
	
	version(ENTICE_0_8_1_PLUGIN)
	{
	}
	else
	{
		/**
			Access a feature: int getFeature(int pluginid, int featureid, void* getinfo, size_t getinfosize);
			Params:
				pluginid = this plugin ID.
				featureid = feature ID; one of EFID_* constants.
				getinfo = pointer to a buffer used by the feature, which may or may not be written to.
				getinfosize = number of bytes at getinfo for use by the feature.
			Returns: defined by the feature; typically 0 on failure.
		**/
		EnticeGetFeatureProc getFeature;
	}
}


const int SHOW_MSG_BOX = 0; /// How a message will be shown.
const int SHOW_MSG_BOX_INFO = 1; /// ditto
const int SHOW_MSG_BOX_ERROR = 2; /// ditto
const int SHOW_MSG_BOX_CONFIRM = 3; /// ditto
const int SHOW_MSG_OUTPUT = 20; /// ditto
version(ENTICE_0_8_1_PLUGIN)
{
}
else
{
	const int SHOW_MSG_INPUT = 40; /// ditto
}


const int EMENU_DEFAULT = 0; /// Preset parent menu IDs.
version(ENTICE_0_8_1_PLUGIN)
{
}
else
{
	const int EMENU_EDIT = -1001; /// ditto
	const int EMENU_SEARCH = -1002; /// ditto
}


version(ENTICE_0_8_1_PLUGIN)
{
}
else
{
	const int EFID_CONFIG = 100; /// Feature IDs.
	const int EFID_CURFILE = 101; /// ditto
	const int EFID_CMDEXEC = 102; /// ditto
	const int EFID_EDITORGETSELTEXT = 103; /// ditto
	const int EFID_EDITORSETSELTEXT = 104; /// ditto
	const int EFID_EXIT = 105; /// ditto
	const int EFID_CURTABTYPE = 106; /// ditto
}


version(ENTICE_0_8_1_PLUGIN)
{
}
else
{
	const int ETAB_NONE = 0; ///
	const int ETAB_UNKNOWN = -1; /// ditto
	const int ETAB_EDITOR = 1; /// ditto
	const int ETAB_DESIGN = 2; /// ditto
}


version(ENTICE_0_8_1_PLUGIN)
{
}
else
{
	const int EF_CONFIG_PLUGIN = 0; ///
	const int EF_CONFIG_CURDIR = 1; /// ditto
}


version(ENTICE_0_8_1_PLUGIN)
{
}
else
{
	///
	struct EF_Config
	{
		EF_Config_GetConfigProc getConfig; ///
		EF_Config_SetConfigProc setConfig; ///
	}
}


///
struct EnticePluginInfo
{
	align(1):
	char* name; // out
	void* gcHandle; // in
	uint flags; // inout
	ubyte curMajorVersion; // in - current plugin arch version.
	ubyte curMinorVersion; // in - ditto.
	ubyte thisMajorVersion; // inout - plugin arch version supported by this plugin.
	ubyte thisMinorVersion; // inout - ditto.
	char* desc; // out - warning: not always available.
}


// Specific plugin versions that match with specific Entice Designer versions.
version(ENTICE_0_8_1_PLUGIN)
{
	const ubyte MAJOR_VERSION = 0;
	const ubyte MINOR_VERSION = 1;
}
else
{
	// Current plugin version.
	const ubyte MAJOR_VERSION = 0;
	const ubyte MINOR_VERSION = 2;
}


private extern(C)
{
	void gc_init();
	void gc_term();
	void _minit();
	void _moduleCtor();
	void _moduleDtor();
	void _moduleUnitTests();
}


import std.gc;
import std.c.windows.windows;
import std.c.stdio;


void beginEnticePlugin(EnticeFunctions* efuncs)
{
	/+
	version(ENTICE_GC)
	{
		std.gc.setGCHandle(epinfo.gcHandle);
		_minit();
		_moduleCtor();
		//_moduleUnitTests();
	}
	+/
	version(ENTICE_GC)
	{
		version(ENTICE_0_8_1_PLUGIN)
		{
			static assert(0);
		}
		//std.gc.setGCHandle(epinfo.gcHandle);
		void* gcHandle;
		efuncs.getFeature(efuncs.id, -9000, &gcHandle, gcHandle.sizeof);
		std.gc.setGCHandle(gcHandle);
	}
	else
	{
		gc_init();
	}
	_minit();
	_moduleCtor();
	//_moduleUnitTests();
}


void endEnticePlugin(EnticeFunctions* efuncs)
{
	/+
	version(ENTICE_GC)
	{
		_moduleDtor();
		std.gc.endGCHandle();
	}
	+/
	_moduleDtor();
	version(ENTICE_GC)
	{
		std.gc.endGCHandle();
	}
	else
	{
		gc_term();
	}
}


extern(Windows) BOOL DllMain(HINSTANCE hInstance, ULONG ulReason, LPVOID lpvReserved)
{
	switch(ulReason)
	{
		case DLL_PROCESS_ATTACH:
			version(ENTICE_GC)
			{
			}
			/+else
			{
				gc_init();
				_minit();
				_moduleCtor();
				//_moduleUnitTests();
			}+/
			break;
		
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
			version(ENTICE_GC)
			{
			}
			else
			{
				//return FALSE; // !
			}
			break;
		
		case DLL_PROCESS_DETACH:
			version(ENTICE_GC)
			{
				version(DigitalMars)
				{
					std.c.stdio._fcloseallp = null;
				}
			}
			/+else
			{
				_moduleDtor();
				gc_term();
			}+/
			break;
		
		default: ;
	}
	
	return TRUE;
}

