import enticeplugin;


export extern(Windows) void enticePluginLoaded(EnticeFunctions* efuncs)
{
	beginEnticePlugin(efuncs); // This call is required for D and must be first.
	
	char[] s;
	
	s = "Compile Command...";
	efuncs.registerMenuItem(efuncs.id, s.ptr, s.length, 42, 0, &_setCompileCommand_menuItem_click);
	
	s = "Compile\tF7";
	efuncs.registerMenuItem(efuncs.id, s.ptr, s.length, 43, 0, &_compile_menuItem_click);
	
	const int KEY_F7 = 118;
	efuncs.registerShortcut(efuncs.id, KEY_F7, 44, &_compile_shortcut_f7);
}


// This function cannot use D's garbage collector.
// Only for load parameters.
// Return one of the following:
// 	0 = Successfully load.
// 	1 = Abort load due to problem.
// 	2 = Successfully unload / prevent load.
export extern(Windows) int enticePluginLoad(EnticePluginInfo* epinfo)
{
	epinfo.name = "Directory Project";
	
	if(epinfo.flags & 2)
		epinfo.desc = "Using directories as simple projects.";
	
	epinfo.thisMajorVersion = MAJOR_VERSION;
	epinfo.thisMinorVersion = MINOR_VERSION;
	
	return 0; // OK.
}


export extern(Windows) void enticePluginUnloaded(EnticeFunctions* efuncs, int n)
{
	// Plugin unloaded code goes here.
	
	endEnticePlugin(efuncs);
}


//import std.process;


const char[] DIRPROJ_CMD_CONFIG = "dirprojcmd";


private void doCompile(EnticeFunctions* efuncs)
{
	char[1024 * 4] buf;
	char[] s;
	int i;
	
	if(!efuncs.getFeature(efuncs.id, EFID_CURFILE, null, 0))
	{
		s = "No files are open";
		efuncs.showMsg(efuncs.id, s.ptr, s.length, SHOW_MSG_BOX_INFO);
		return;
	}
	
	EF_Config config;
	if(!efuncs.getFeature(efuncs.id, EFID_CONFIG, &config, config.sizeof))
	{
		s = "Unable to interface with config";
		efuncs.showMsg(efuncs.id, s.ptr, s.length, SHOW_MSG_BOX_ERROR);
		return;
	}
	
	i = config.getConfig(efuncs.id, EF_CONFIG_CURDIR, DIRPROJ_CMD_CONFIG.ptr, DIRPROJ_CMD_CONFIG.length, buf.ptr, buf.length);
	if(!i)
	{
		s = "No compile command is set for this directory";
		//efuncs.showMsg(efuncs.id, s.ptr, s.length, SHOW_MSG_BOX_INFO);
		efuncs.showMsg(efuncs.id, s.ptr, s.length, SHOW_MSG_OUTPUT);
		return;
	}
	
	if(i > buf.length)
		i = buf.length;
	s = buf[0 .. i];
	
	//efuncs.showMsg(efuncs.id, s.ptr, s.length, 0);
	//std.process.system(s); // Doesn't set current dir, locks up the GUI, ...
	if(!efuncs.getFeature(efuncs.id, EFID_CMDEXEC, s.ptr, s.length))
	{
		s = "Unable to execute command";
		efuncs.showMsg(efuncs.id, s.ptr, s.length, SHOW_MSG_BOX_ERROR);
		return;
	}
}


private void doSetCompileCommand(EnticeFunctions* efuncs)
{
	char[1024 * 4] buf;
	char[] s;
	int i, imax;
	
	if(!efuncs.getFeature(efuncs.id, EFID_CURFILE, null, 0))
	{
		s = "No files are open";
		efuncs.showMsg(efuncs.id, s.ptr, s.length, SHOW_MSG_BOX_INFO);
		return;
	}
	
	s = "Enter a compile command for the current file's directory:";
	buf[0 .. s.length] = s;
	buf[s.length] = 0;
	
	EF_Config config;
	if(!efuncs.getFeature(efuncs.id, EFID_CONFIG, &config, config.sizeof))
	{
		s = "Unable to interface with config";
		efuncs.showMsg(efuncs.id, s.ptr, s.length, SHOW_MSG_BOX_ERROR);
		return;
	}
	
	imax = buf.length - s.length - 1;
	i = config.getConfig(efuncs.id, EF_CONFIG_CURDIR, DIRPROJ_CMD_CONFIG.ptr, DIRPROJ_CMD_CONFIG.length, buf.ptr + s.length + 1, imax);
	if(i)
	{
		if(i > imax)
			i = imax;
		buf[s.length] = 1;
		buf[s.length + 1 + i] = 0;
	}
	
	i = efuncs.showMsg(efuncs.id, buf.ptr, buf.length, SHOW_MSG_INPUT);
	if(!i && buf[0] == 0xFE) // Cancel.
		return;
	if(i > buf.length)
		i = buf.length;
	s = buf[0 .. i];
	//s = std.string.strip(s);
	
	if(!config.setConfig(efuncs.id, EF_CONFIG_CURDIR, DIRPROJ_CMD_CONFIG.ptr, DIRPROJ_CMD_CONFIG.length, s.ptr, s.length))
	{
		s = "Unable to save config 'compile command'";
		efuncs.showMsg(efuncs.id, s.ptr, s.length, SHOW_MSG_BOX_ERROR);
		return;
	}
}


private extern(Windows) void _compile_shortcut_f7(EnticeFunctions* efuncs, int id)
{
	doCompile(efuncs);
}


private extern(Windows) void _compile_menuItem_click(EnticeFunctions* efuncs, int id)
{
	doCompile(efuncs);
}


private extern(Windows) void _setCompileCommand_menuItem_click(EnticeFunctions* efuncs, int id)
{
	doSetCompileCommand(efuncs);
}

